#include <stdio.h>
#include <string.h>
#include <gd.h>
#include <limits.h>

#include "dpf.h"

static DPFHANDLE picframe;

static int sw=0,sh=0; // screen width, height
static unsigned char *pixels=NULL; // pixel buffer that is copied to device

enum
{
	FILETYPE_PNG	= 1,
	FILETYPE_JPG	= 2,
	FILETYPE_GIF	= 3,
};

enum
{
	ACTION_IMAGE	= 1,
	ACTION_TEXT	= 2,
	ACTION_COLOR	= 3,
};

static void check_for_output_device()
{
	char *dev="usb0";
	int r=dpf_open(dev,&picframe);

	if (picframe) 
	{ 
		sw=picframe->width;
		sh=picframe->height;
		pixels=(unsigned char*)malloc(sw*sh*2);
		fprintf(stderr,"INFO: successfully connected to picframe\n");
	}
	else
		fprintf(stderr,"WARN: failed to connect to picframe\n");
}

int draw_image(char* filename, int filetype, int resize, int fill)
{
	FILE *in;
	gdImagePtr im, im_orig;

	int x,y,b1,b2,cw=sw,ch=sh,oh=0,ow=0;

	unsigned int c,r,g,b;
    
	unsigned char *p = pixels;
    
	in=fopen(filename,"rb");

	if (in==NULL)
	{
		fprintf(stdout,"No such file\n");
		return 0;
	}

	if(filetype == FILETYPE_PNG)
		im_orig=gdImageCreateFromPng(in);
	else if(filetype == FILETYPE_JPG)
		im_orig=gdImageCreateFromJpeg(in);
	else if(filetype == FILETYPE_JPG)
		im_orig=gdImageCreateFromGif(in);

	fclose(in);

	if(resize==0 && fill==0 || (im_orig->sx == sw && im_orig->sy == sh))
	{
		im = im_orig;
	}
	else
	{
		if(fill == 0)
		{
			if(im_orig->sx >= sw && im_orig->sy >= sh)
			{
				if((im_orig->sx)/sw >= (im_orig->sy)/sh)
				{
					cw = sw;
					ch = (im_orig->sy)*sw/(im_orig->sx);
					oh = (sh - ch)/2;

				}
				else
				{
					ch = sh;
					cw = (im_orig->sx)*sh/(im_orig->sy);
					ow = (sw - cw)/2;
				}

			}
/*
TODO	else: scale up images that are smaller than the screen size
	Is this neccessary?  Images to be displayed are rarely smalller
	than DPF screen. fwiw, --fill mode will scale up small images
	non-proportionally, at least.
*/
		}


		im = gdImageCreateTrueColor(sw, sh);
		gdImageCopyResampled(im, im_orig, ow, oh, 0, 0, cw, ch, im_orig->sx, im_orig->sy);
	}

	if (im==NULL)
	{
		printf("%s: not a png-file.\n",filename);
		return 0;
	}

	for (y=0; y<sh; y++)
	{
		for (x=0; x<sw; x++)
		{
			c = gdImageGetPixel(im, x, y);
			if (gdImageTrueColor(im))
			{
				r=gdTrueColorGetRed(c);
				g=gdTrueColorGetGreen(c);
				b=gdTrueColorGetBlue(c);
			}
			else
			{
				r=gdImageRed(im,c);
				g=gdImageGreen(im,c);
				b=gdImageBlue(im,c);
			}

			b1 = ( ( ( (r >> 3) << 3) | (g >> 5) ) );
			b2 = ( (g & 28) << 3 ) | (b >> 3);

			*p++ = b1;
			*p++ = b2;
		}
	}

	gdImageDestroy(im);
	return 1;
}

void draw_color(int color)
{
	int r,g,b,i,j,d;
	uint8_t b1,b2;
	unsigned char *p = pixels;
	unsigned char *p2 = pixels;// + 128*128*2;

	b = color % 1000;
	color /= 1000;
	g = color % 1000;
	color /= 1000;
	r = color % 1000;
	color /= 1000;

	fprintf(stdout, "r: %d, g: %d, b: %d\n",r,g,b);

	b1 = ( ( ( (r >> 3) << 3) | (g >> 5) ) );
	b2 = ( (g & 28) << 3 ) | (b >> 3);


	for(i=1; i<=sh; i++)
	{
		for(j=1; j<=(sw); j++)
		{
			*p = b1;
			p++;
			*p = b2;
			p++;
		}

	}
}

char *get_filename_ext(char *filename) {
	char *dot = strrchr(filename, '.');
    if(!dot || dot == filename) return "";
    return dot + 1;
}

int determine_filetype(char* str)
{

	char a;

	strcpy(str, get_filename_ext(str));

	int i;

	for (i = 0; str[i] != '\0'; i++)
	{
		a = tolower(str[i]);
		str[i] = a;
	}

	if(strcmp (str,"png") == 0)
	{
		fprintf(stdout,"Filetype: png\n");
		return FILETYPE_PNG;
	}
	else if(strcmp (str,"jpg") == 0)
	{
		fprintf(stdout,"Filetype: jpg\n");
		return FILETYPE_JPG;
	}
	else if(strcmp (str,"jpeg") == 0)
	{
		fprintf(stdout,"Filetype: jpg\n");
		return FILETYPE_JPG;
	}
	else if(strcmp (str,"gif") == 0)
	{
		fprintf(stdout,"Filetype: gif\n");
		return FILETYPE_GIF;
	}
	else
	{
		fprintf(stdout,"Unsupported file extension, default to png...\n");
		return FILETYPE_PNG;
	}

return 1;

}

void print_help()
{
	fprintf(stdout,"dpfsend v0.1\n");
	fprintf(stdout,"Options:\n");
	fprintf(stdout,"\t-i, --image [filename]\t");
	fprintf(stdout,"Display image\n");

	fprintf(stdout,"\t-r, --resize\t\t");
	fprintf(stdout,"Resize image\n");

	fprintf(stdout,"\t-f, --fill\t\t");
	fprintf(stdout,"Fill screen with image\n\n");

	fprintf(stdout,"\t-c, --color [value]\t");
	fprintf(stdout,"fill screen with solid color RRRGGGBBB\n");

	fprintf(stdout,"\t-t, --text [value]\t");
	fprintf(stdout,"Display text (not implemented)\n");

	fprintf(stdout,"\t-h, --help\t\t");
	fprintf(stdout,"This help text\n");

	fprintf(stdout,"\nSupported formats: png, jpeg, gif\n");
}

int main(int argc, char **argv)
{

	int action=0, i,filetype, myobject=0;
	char *mystring;
	char str[PATH_MAX] ="";
	int resize=0;
	int fill=0;

	for(i=0; i<argc; i++)
	{

		if(strcmp(argv[i],"-i") == 0 || strcmp(argv[i],"--image") == 0)
		{
			myobject=i+1;
			action = ACTION_IMAGE;
			i++;
		}
		else if(strcmp(argv[i],"-t") == 0 || strcmp(argv[i],"--text") == 0)
		{
			myobject=i+1;
			action = ACTION_TEXT;
			i++;
		}
		else if(strcmp(argv[i],"-c") == 0 || strcmp(argv[i],"--color") == 0)
		{
			myobject=i+1;
			action = ACTION_COLOR;
		}
		else if(strcmp(argv[i],"-r") == 0 || strcmp(argv[i],"--resize") == 0)
		{
			resize = 1;
		}
		else if(strcmp(argv[i],"-f") == 0 || strcmp(argv[i],"--fill") == 0)
		{
			fill = 1;
		}
		else if(strcmp(argv[i],"-h") == 0 || strcmp(argv[i],"--help") == 0)
		{
			print_help();
			return 0;
		}
	}

	if(myobject == 0)
	{
		if(argc == 2)
		{
			fprintf(stdout,"Assuming default options....\n");
			myobject=1;
			action = ACTION_IMAGE;
		}
	}

	if(myobject >= argc)
	{
		return 1;
	}
	else
	{
		strcpy(str, argv[myobject]);
	}

	if(action)
	        check_for_output_device();
	else
	{
		print_help();
		return 1;
	}

	if(action == ACTION_IMAGE)
	{
		filetype = determine_filetype(str);

		draw_image(argv[myobject], filetype, resize, fill);
	}
	else if(action == ACTION_TEXT)
	{
		// not implemented
		fprintf(stderr,"Not implemented\n");
		print_help();
	}
	else if(action == ACTION_COLOR)
	{
		fprintf(stdout,"Color....\n");
		draw_color(atoi(argv[myobject]));
	}
	else
	{
		fprintf(stderr,"Do nothing....\n");
		print_help();
	}

	

	short rect[4];
	rect[0]=0;
	rect[1]=0;
	rect[2]=sw;
	rect[3]=sh;	
	dpf_screen_blit(picframe,pixels,rect);

	return 0;
}
